// minimal convolver by jl 2003
// Version 1.0, 2003-05-28

#include "stdafx.h"
#include <iostream>
#include <fstream>
#include <cmath>
using namespace std;

const double PI = 4.0*atan(1.0);

#pragma pack(1) // tell MSVC not to leave gaps in data structures
struct wavHeader
{
	// We assume that short has 16 bits and long has 32 bits.
	unsigned long riffString; // 'FFIR' (litte endian!) 
	long fileLength; // length - 8
	unsigned long waveString; // 'EVAW'
	unsigned long fmtString; // ' tmf'
	long fmtLength; // 16
	short formatTag; // 1 = PCM
	short channels; // 1, 2, ...
	long sampleRate; // samples per second, e.g. 44100
	long bytesPerSecond; // sampleRate*blockAlign
	short blockAlign; // (channels*bitsPerSample)/8
	short bitsPerSample; // 8, 16, ...
	unsigned long dataString; // 'atad'
	long dataLength;
};
#pragma pack()
// Don't forget: On big endian systems (i.e. Macs) we have to shuffle bytes!

int main(int argc, char* argv[])
{
	ifstream inFile("in.wav", ios::in | ios::binary);
	if(!inFile)
	{
		cerr << "Can't open in.wav" << endl;
		exit(-1);
	}

	wavHeader inHeader;
	inFile.read((char*) &inHeader, sizeof(inHeader));

	// code to add: check if header is filled as expected
	// code to add: check for file errors

	if(inHeader.channels != 1 || inHeader.bitsPerSample != 16)
	{
		cerr << "in.wav: no standard 16 bit mono file" << endl;
		exit(-2);
	}

	long numberOfSamples = inHeader.dataLength/2;
	short *samples = new short[numberOfSamples];
	inFile.read((char*) samples, inHeader.dataLength);

	// code to add: check for file errors

	ifstream impResFile("imp_res.wav", ios::in | ios::binary);
	if(!impResFile)
	{
		cerr << "Can't open imp_res.wav" << endl;
		exit(-1);
	}

	wavHeader impResHeader;
	impResFile.read((char*) &impResHeader, sizeof(impResHeader));

	// code to add: check if header is filled as expected
	// code to add: check for file errors
	// code to add: check that sampling rates are equal

	if(impResHeader.channels != 2 || impResHeader.bitsPerSample != 16)
	{
		cerr << "imp_res.wav: no standard 16 bit stereo file" << endl;
		exit(-2);
	}

	long numberOfImpResSamples = impResHeader.dataLength/4;
	short (*impResSamples)[2] = new short[numberOfImpResSamples][2];
	impResFile.read((char*) impResSamples, impResHeader.dataLength);

	// code to add: check for file errors

	cout << "You may need to adjust the level to avoid clipping." << endl;
	const double level = 0.03;

	short (*outSamples)[2] = new short[numberOfSamples][2];

	int i;
	for(i=0; i<numberOfSamples; ++i)
	{
		if(i%500 == 0)
		{
			cout << i << " samples of " << numberOfSamples << " processed." << endl;
		}

		double resultL = 0.0; // range -1.0 ... +1.0
		double resultR = 0.0; // range -1.0 ... +1.0

		int j;
		for(j=0; j<numberOfImpResSamples; ++j)
		{
			double impulseL = impResSamples[j][0]/(double)0x7FFF;
			double impulseR = impResSamples[j][1]/(double)0x7FFF;
			if (i-j < 0) // better solution: limit the range of j in "for"
				continue;
			double samp = level*samples[i-j]/(double)0x7FFF;
			resultL += impulseL*samp;
			resultR += impulseR*samp;
		}

		// round-off isn't treated right;
		// clipping isn't handled at all
		outSamples[i][0] = (short)(0x7FFF*resultL);
		outSamples[i][1] = (short)(0x7FFF*resultR);
	}

	ofstream outFile("out.wav", ios::out | ios::binary | ios::trunc);
	
	// somewhat ugly reuse of inHeader
	inHeader.channels = 2;
	inHeader.dataLength = numberOfSamples*4;
	inHeader.fileLength = inHeader.dataLength - 8;
	outFile.write((char*) &inHeader, sizeof(inHeader));
	outFile.write((char*) outSamples, inHeader.dataLength);		

	// code to add: check for file errors

	delete[] outSamples;
	delete[] impResSamples;
	delete[] samples;

	return 0;
}